//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalAtomsMisc.h --- miscellaneous Cal atoms
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1997-10-25: 1.0.0: - skeleton
//		UL, 1997-??-??: 1.1.0: - extended
//		UL, 1998-08-28: 1.2.0: - EnergyCorrector class added
//		UL, 2000-04-25: 1.3.0: - additional mode parameter data
//		UL, 2002-06-14: 1.4.0: - phased Background out into separate unit
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/cal/src/CalAtomsMisc.h,v 1.7 2003/10/03 14:15:40 sasbuild Exp $
//

#ifndef CAL_ATOMS_MISC_H
#define CAL_ATOMS_MISC_H

#include "Ccf.h"
#include "CalAtomsBase.h"
#include "Celestial.h"

#include <vector>

//
//	---( mode parameters )------------------------------------------------
//
class ModeParameters : public CalAtom {
public:
	friend class CalAtomCollection<ModeParameters>;

	//	default constructor - does nothing
	ModeParameters() {}

	static unsigned id() { return Ccf::ModeParam; }

	float integrationTime() const { return _integrationTime; }
	float shiftTime() const { return _shiftTime; }
	float frameTime() const { return _frameTime; }
	float cycleTime() const { return _cycleTime; }
	float timeResolution() const { return _timeResolution; }
	float dutyCycle() const { return _dutyCycle; }
	float smearedFraction() const { return _smearedFraction; }
	float pointSourcePileUp() const { return _pointSourcePileUp; }
	float limitingFlux() const { return _limitingFlux; }
	short underX() const { return _underX; }
	short overX() const { return _overX; }
	short underY() const { return _underY; }
	short overY() const { return _overY; }
	short windowX0() const { return _windowX0; }
	short windowY0() const { return _windowY0; }
	short windowDX() const { return _windowDX; }
	short windowDY() const { return _windowDY; }

	~ModeParameters() {}

private:
	float _integrationTime,
		  _shiftTime,
		  _frameTime,
		  _cycleTime,
		  _timeResolution,
		  _dutyCycle,
		  _smearedFraction,
		  _pointSourcePileUp,
		  _limitingFlux;
	short _underX,
		  _overX,
		  _underY,
		  _overY,
		  _windowX0,
		  _windowY0,
		  _windowDX,
		  _windowDY;
};


//
//	--- ( HK windows )-----------------------------------------------------
//
class HkWindows : public CalAtom {
public:
	friend class CalAtomCollection<HkWindows>;

	//	default constructor - does nothing
	HkWindows() {}

	void dump() const;

	~HkWindows() {}

private:
	//	data members
};

#endif

