//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	PatternLibServer --- the patter library server class
//
//	DESCRIPTION:
//		Instances of class PatternLibServer give access to the event pattern
//		library.
//
//	CHANGE HISTORY:
//		UL, 2000-05-18: 1.0.0: - first incarnation (from CalAtomsEmos)
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/cal/src/PatternLibServer.h,v 1.7 2003/10/03 14:15:41 sasbuild Exp $
//

#ifndef PATTERN_LIB_SERVER_H
#define PATTERN_LIB_SERVER_H

#include "CalAtomsBase.h"
#include "CalAtomsMisc.h"
#include "Image.h"

//
//	---( EventPattern )--------------------------------------------------------
//
class EventPattern : public Image<int8> {
public:
	EventPattern(unsigned short id,
				 unsigned xExtent, unsigned yExtent, const int8 *vector);
		//
		//	PURPOSE:
		//		Construct event pattern with id <id> of size
		//		<xExtent> x <yExtent> from flat data <vector>
		//


	unsigned short id() const { return _id; }
	unsigned short grade() const { return _grade; }
		//
		//	PURPOSE:
		//		Return pattern id/grade
		//

	~EventPattern() {}

private:
	unsigned short _id, _grade;
}; // EventPattern

//
//	---( PatternLibServer )----------------------------------------------------
//
class PatternLibServer : public CalAtom,
	 public virtual CalChild<CalAtomCollection<PatternLibServer> > {
public:
	PatternLibServer(const Table *table,
					CalAtomCollection<PatternLibServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate PatternLibServer object from data set
		//


	virtual void dump(std::ostream &) {}
		//
		//	PURPOSE:
		//		Dump data members to an ostream
		//
		//	ARGUMENTS:
		//		In:
		//			os:	stream to print on
		//


	static unsigned id() { return Ccf::PatternLib; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//


	virtual unsigned patterns() const { return _patternMap.size(); }
		//
		//	PURPOSE:
		//		Return number of patterns
		//


	virtual const EventPattern &pattern(unsigned patternId) const;
		//
		//	PURPOSE:
		//		Return pattern with ID <patternId>; <patternId> is
		//		a number in the range [0, PatternLib::patterns()-1]
		//


	virtual std::vector<std::vector<unsigned short> > gradePatternList() const;
		//
		//	PURPOSE:
		//		Return the list of pattern per grade <res>:
		//			<res>[n] is a vector with all grade-n patterns
		//


	virtual ~PatternLibServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	typedef std::map<unsigned, EventPattern *, std::less<unsigned> > MapType;
	MapType _patternMap;
};

#endif


